<?php

declare(strict_types=1);

namespace Drupal\monolog\Logger\Handler;

use Drupal\Core\Logger\RfcLogLevel;
use Monolog\Handler\AbstractProcessingHandler;
use Monolog\Level;
use Monolog\LogRecord;
use Psr\Log\LoggerInterface;

/**
 * Forwards logs to a Drupal logger.
 */
class DrupalHandler extends AbstractProcessingHandler {

  /**
   * The wrapped Drupal logger.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected LoggerInterface $logger;

  /**
   * Constructs a Default object.
   *
   * @param \Psr\Log\LoggerInterface $wrapped
   *   The wrapped Drupal logger.
   * @param int|string $level
   *   The minimum logging level at which this handler will be triggered.
   * @param bool $bubble
   *   Whether the messages that are handled can bubble up the stack or not.
   */
  public function __construct(
    LoggerInterface $wrapped,
    $level = Level::Debug,
    bool $bubble = TRUE
  ) {
    parent::__construct($level, $bubble);

    $this->logger = $wrapped;
  }

  /**
   * {@inheritdoc}
   */
  public function write(LogRecord $record): void {
    // Set up context with the data Drupal loggers expect.
    // @see Drupal\Core\Logger\LoggerChannel::log()

    $extra = $record->extra;
    $context = $record['context'] + ($record['drupal_context_placeholders'] ?? []) + [
      'channel' => $record->channel,
      'link' => '',
      'user' => $extra['user'] ?? NULL,
      'uid' => $extra['uid'] ?? 0,
      'request_uri' => $extra['request_uri'] ?? '',
      'referer' => $extra['referer'] ?? '',
      'ip' => $extra['ip'] ?? 0,
      'timestamp' => $record->datetime->format('U'),
    ];

    switch ($record->level) {
      case Level::Emergency:
        $rfc_level = RfcLogLevel::EMERGENCY;
        break;
      case Level::Alert:
        $rfc_level = RfcLogLevel::ALERT;
        break;
      case Level::Critical:
        $rfc_level = RfcLogLevel::CRITICAL;
        break;
      case Level::Error:
        $rfc_level = RfcLogLevel::ERROR;
        break;
      case Level::Warning:
        $rfc_level = RfcLogLevel::WARNING;
        break;
      case Level::Notice:
        $rfc_level = RfcLogLevel::NOTICE;
        break;
      case Level::Info:
        $rfc_level = RfcLogLevel::INFO;
        break;
      case Level::Debug:
        $rfc_level = RfcLogLevel::DEBUG;
        break;
    }

    $this->logger->log($rfc_level, $record['drupal_message'] ?? $record->message, $context);
  }

}
