<?php

declare(strict_types=1);

namespace Drupal\monolog\Logger;

use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\Core\Logger\RfcLogLevel;
use Drupal\Core\Session\AccountInterface;
use Monolog\Level;
use Monolog\Logger as BaseLogger;
use Psr\Log\LoggerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Logger class for the Drupal Monolog module.
 *
 * Allows the channel to be modified after the class is instantiated. This is
 * normally not a good idea, but it is necessary to reconcile the differences in
 * the Monolog library and how the watchdog type relates to the logging
 * facility.
 */
class Logger extends BaseLogger implements LoggerChannelInterface {

  /**
   * {@inheritdoc}
   */
  public function addRecord(int|Level $level, string $message, array $context = []): bool {
    if (in_array($level, array_keys(RfcLogLevel::getLevels()))) {
      switch ($level) {
        case RfcLogLevel::EMERGENCY:
          $level = Level::Emergency;
          break;
        case RfcLogLevel::ALERT:
          $level = Level::Alert;
          break;
        case RfcLogLevel::CRITICAL:
          $level = Level::Critical;
          break;
        case RfcLogLevel::ERROR:
          $level = Level::Error;
          break;
        case RfcLogLevel::WARNING:
          $level = Level::Warning;
          break;
        case RfcLogLevel::NOTICE:
          $level = Level::Notice;
          break;
        case RfcLogLevel::INFO:
          $level = Level::Info;
          break;
        case RfcLogLevel::DEBUG:
          $level = Level::Debug;
          break;
      }
    }

    return parent::addRecord($this->toMonologLevel($level), $message, $context);
  }

  /**
   * {@inheritdoc}
   */
  public function setRequestStack(RequestStack $requestStack = NULL) {
    // Do nothing, use a handler for this.
  }

  /**
   * {@inheritdoc}
   */
  public function setCurrentUser(AccountInterface $current_user = NULL) {
    // Do nothing, use a handler for this.
  }

  /**
   * {@inheritdoc}
   */
  public function setLoggers(array $loggers) {
    // Do nothing.
  }

  /**
   * {@inheritdoc}
   */
  public function addLogger(LoggerInterface $logger, $priority = 0) {
    // Do nothing.
  }

}
